-----------------------------------------------------------------------------------------------
-- Client Lua Script for VideoProfiles
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
require "Window"
require "GameLib"
require "Apollo"
-----------------------------------------------------------------------------------------------
-- VideoProfiles Module Definition
-----------------------------------------------------------------------------------------------
local VideoProfiles = {} 
---------------------------------------------------------------------------------------------------
local tFrames = {"video.framerateMax", "video.verticalSync", "lod.viewDistance", "lod.farFogDistance", "lod.clutterDistance", "camera.fovY", "ppp.gamma", "lod.textureFilter", "lod.textureLodMin", "lod.shadowMapSize", "fxaa.preset", "lod.clutterDensity", "world.propScreenHeightPercentMin", "particle.envParticleScale", "spell.visualSuppression", "lod.landLod", "lod.renderTargetScale", "camera.shake", "camera.reorient", "fxaa.enable", "draw.shadows"} 
-----------------------------------------------------------------------------------------------------
function VideoProfiles:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end

function VideoProfiles:Init()
	Apollo.RegisterAddon(self, true, "VideoProfiles", nil)
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
end
-----------------------------------------------------------------------------------------------
-- VideoProfiles OnLoad
-----------------------------------------------------------------------------------------------
function VideoProfiles:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("VideoProfiles.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end
-----------------------------------------------------------------------------------------------
-- VideoProfiles OnDocLoaded
-----------------------------------------------------------------------------------------------
function VideoProfiles:OnDocLoaded()
    if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "VideoProfilesForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.wndMain:Show(false, true)
		self.wndProfiles = Apollo.LoadForm(self.xmlDoc, "ProfilesForm", nil, self)
		self.wndProfiles:Show(false, true)
		self.wndReload = Apollo.LoadForm(self.xmlDoc, "ReloadForm", nil, self)
		self.wndReload:Show(false, true)
		self.xmlDoc = nil
		if self.locSavedWindowLoc then
			self.wndMain:MoveToLocation(self.locSavedWindowLoc)
		end
		if self.locProfilesLocation then
			self.wndProfiles:MoveToLocation(self.locProfilesLocation)
		end
		Apollo.RegisterSlashCommand("vp", "OnOptions", self)
		Apollo.RegisterEventHandler("OnVideoProfilesOptions", "OnOptions", self)
        self.wndList = self.wndMain:FindChild("ScrollList")
		self.timeOn = ApolloTimer.Create(2, self.bShow, "OnStart", self)
		self.timestart = ApolloTimer.Create(4, self.bShow, "OnProfiles", self)
	end
end
------------ Register Addon in Menu ---------
function VideoProfiles:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Video Profiles", {"OnVideoProfilesOptions", "", "TargetFrameSprites:sprRoleSummoner"})
end
-----------------------------------------------------------------------------
function VideoProfiles:OnStart()
    self.timeOn = nil
	self.tMonitor = Apollo.GetConsoleVariable("video.exclusiveDisplayMode")
    local unitPlayer = GameLib.GetPlayerUnit()
	local strGuildName = unitPlayer:GetGuildName() or ""
    if Apollo.GetAddon("Gorynych Reborn") ~= nil or strGuildName == "The Core" or strGuildName == "Star Bears" then
	    self.bRus = true
	else self.bRus = false 
	end
	self.wnd1 = self.wndProfiles:FindChild("Profile1")
    self.wnd2 = self.wndProfiles:FindChild("Profile2")
    self.wnd3 = self.wndProfiles:FindChild("Profile3")
	local x = self.tMonitor.x/2
	local y = self.tMonitor.y/2
	if self.tMonitor.y < 1025 then self.wndMain:SetScale(0.95) end
	if self.tMonitor.y < 1000 then self.wndMain:SetScale(0.9) end
	if self.tMonitor.y < 960 then self.wndMain:SetScale(0.85) end
	self.wndReload:SetAnchorOffsets(x-80,y-300,x+280,y-100)
end

function VideoProfiles:OnProfiles()
    self.timestart = nil
	if self.bShow == false then return end
	if self.tProfiles == nil then self.tProfiles = {}
	    for n = 1, 3 do
		    local tCurr = {}
	        for idx, name in pairs(tFrames) do
			    tCurr[name] = Apollo.GetConsoleVariable(name)
			end
			self.tProfiles[n] = tCurr
		end
	end
    self.wndProfiles:Show(true)
	self.wndProfiles:SetScale(0.9)
	if self.nCurrentProfile == nil then self.nCurrentProfile = 1 end
	self.wndProfiles:FindChild("Profile"..self.nCurrentProfile):SetCheck(self.wndProfiles:IsVisible())
	self.wndProfiles:FindChild("Profile"..self.nCurrentProfile):FindChild("Numb"..self.nCurrentProfile):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Magic"..self.nCurrentProfile)
end

function VideoProfiles:OnProfile()
    if self.wnd1:IsChecked() then self.nCurrentProfile = 1
	    self.wnd1:FindChild("Numb1"):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Magic1")
		self.wnd2:FindChild("Numb2"):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Vulnerable2")
		self.wnd3:FindChild("Numb3"):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Vulnerable3")
	end
	if self.wnd2:IsChecked() then self.nCurrentProfile = 2
	    self.wnd2:FindChild("Numb2"):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Magic2")
	    self.wnd1:FindChild("Numb1"):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Vulnerable1")
	    self.wnd3:FindChild("Numb3"):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Vulnerable3")
	end
    if self.wnd3:IsChecked() then self.nCurrentProfile = 3
	    self.wnd3:FindChild("Numb3"):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Magic3")
	    self.wnd1:FindChild("Numb1"):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Vulnerable1")
	    self.wnd2:FindChild("Numb2"):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Vulnerable2")
	end
	self.wndProfiles:FindChild("Profile"..self.nCurrentProfile):SetCheck(self.wndProfiles:IsVisible())
	self.wndProfiles:FindChild("Profile"..self.nCurrentProfile):FindChild("Numb"..self.nCurrentProfile):SetSprite("CRB_CritNumberFloaters:sprCritNumber_Magic"..self.nCurrentProfile)
	for name, data in pairs(self.tProfiles[self.nCurrentProfile]) do
	    Apollo.SetConsoleVariable(name, data)
	end
	if self.wndMain:IsVisible() then self:OnVideoProfilesOn() end
	Sound.Play(77)
end
-----------------------------------------------------------------------------------------------
-- VideoProfiles Functions
-----------------------------------------------------------------------------------------------
function VideoProfiles:OnVideoProfilesOn()
	self.wndMain:Invoke() -- show the window
	self.wndMain:FindChild("CurrentProfele"):SetText("Profile "..self.nCurrentProfile)
	for idx, name in pairs(tFrames) do
	    if idx > 19 then break end
	    local fValue = Apollo.GetConsoleVariable(name)
		if fValue == false or fValue == true then 
		    self.wndList:FindChild(name):SetCheck(Apollo.GetConsoleVariable(name) == true)
		else
		    if tonumber(fValue) ~= nil and fValue ~= math.modf(fValue) then
			    fValue = tonumber(string.format("%.02f", fValue))
			end
			if name == "lod.textureLodMin" then fValue = 10 - fValue end
			if name == "spell.visualSuppression" then fValue = 1 - fValue end  
			if name == "lod.shadowMapSize" and Apollo.GetConsoleVariable("draw.shadows") == false then fValue = 0 end
			if name == "lod.clutterDensity" then fValue = fValue + 1 end
	        self.wndList:FindChild("Indication_"..name):SetText(fValue)
			if name == "world.propScreenHeightPercentMin" then fValue = fValue * (-1)  end
			self.wndList:FindChild(name):SetValue(fValue)
			if name == "particle.envParticleScale" then self.wndList:FindChild("Indication_"..name):SetText(fValue*100) end
			if name == "spell.visualSuppression" then self.wndList:FindChild("Indication_"..name):SetText(fValue*10) end
			if name == "lod.landLod" then 
			    if fValue == 0 then fValue = "Low" else fValue = "Normal" end
			    self.wndList:FindChild("Indication_"..name):SetText(fValue) 
			end
			if name == "lod.textureFilter" then
			    if fValue == 2 then fValue = "Anisotropic" 
				elseif fValue == 1 then fValue = "Trilinear"
				else fValue = "Bilinear" end
				self.wndList:FindChild("Indication_"..name):SetText(fValue)
			end
			if name == "video.framerateMax" then
		        if Apollo.GetConsoleVariable("video.verticalSync") == true then
		            self.wndList:FindChild("VerticalSyncBLocker"):Show(true)
		        else self.wndList:FindChild("VerticalSyncBLocker"):Show(false) end
		    end
		end
	end
	if self.bCheckRu or self.bRus then self.wndMain:FindChild("Ru"):SetCheck(self.bCheckRu == true or self.bRus == true) end
	if self.bShow == nil then self.bShow = true end
	if self.bShow then self.wndMain:FindChild("ShowProfiles"):SetCheck(self.bShow == true) end
	if self.bCheckRu == true or self.bRus == true then self:OnLocalizationRu() end		
end

function VideoProfiles:OnOptionsScrolling(wndHandler, wndControl, fValue, fOldValue)
    local name = wndControl:GetName()
	if fValue ~= math.modf(fValue) then
		fValue = tonumber(string.format("%.02f", fValue))
	end
	if name == "lod.shadowMapSize" then 
	    if fValue > 0 then  Apollo.SetConsoleVariable("draw.shadows", true)
		else Apollo.SetConsoleVariable("draw.shadows", false) end
		if fValue == 3072 then
    		if fValue > fOldValue then fValue = 4096 else fValue = 2048 end
			self.wndList:FindChild(name):SetValue(fValue)
		end
	end
	if name == "world.propScreenHeightPercentMin" then fValue = fValue * (-1)  end
	if name == "fxaa.preset" and fValue == 0 then 
	    Apollo.SetConsoleVariable("fxaa.enable", false) end
	if name == "fxaa.preset" and fValue > 0 then 
	    Apollo.SetConsoleVariable("fxaa.enable", true) end
	self.wndList:FindChild("Indication_"..name):SetText(fValue)
	if name == "spell.visualSuppression" then fValue = 1 - fValue end  
	if name == "lod.textureLodMin" then fValue = 10 - fValue end
	if name == "lod.clutterDensity" then fValue = fValue - 1 end
    Apollo.SetConsoleVariable(name, fValue)
	if name == "particle.envParticleScale" then 
	    self.wndList:FindChild("Indication_"..name):SetText(fValue*100) end
	if name == "spell.visualSuppression" then 
	    self.wndList:FindChild("Indication_"..name):SetText(10-fValue*10) end
	if name == "lod.landLod" then 
		if fValue == 0 then fValue = "Low" else fValue = "Normal" end
		self.wndList:FindChild("Indication_"..name):SetText(fValue) 
	end
	if name == "lod.textureFilter" then
		if fValue == 2 then fValue = "Anisotropic" 
		elseif fValue == 1 then fValue = "Trilinear"
		else fValue = "Bilinear" end
		self.wndList:FindChild("Indication_"..name):SetText(fValue)
	end
	if self.wndMain:IsVisible() then self:OnVideoProfilesOn() end
end

function VideoProfiles:OnOptionsCheckbox()
    if self.wndList:FindChild("video.verticalSync"):IsChecked() then 
	    Apollo.SetConsoleVariable("video.verticalSync", true)
    else Apollo.SetConsoleVariable("video.verticalSync", false) end
    if self.wndList:FindChild("camera.reorient"):IsChecked() then 
	    Apollo.SetConsoleVariable("camera.reorient", true)
    else Apollo.SetConsoleVariable("camera.reorient", false) end
    if self.wndList:FindChild("camera.shake"):IsChecked() then 
	    Apollo.SetConsoleVariable("camera.shake", true)
    else Apollo.SetConsoleVariable("camera.shake", false) end
	if self.wndMain:FindChild("Ru"):IsChecked() then self.bCheckRu = true
    else self.bCheckRu = false
	    if self.bRus == false then self.wndReload:Show(true) end
   	end
	if self.wndMain:FindChild("ShowProfiles"):IsChecked() then self.bShow = true 
	    self:OnProfiles()
	end
	if self.wndMain:IsVisible() then self:OnVideoProfilesOn() end
end

function VideoProfiles:OnCloseProfiles()
    self.bShow = false
	self.wndProfiles:Show(false)
end

function VideoProfiles:OnCloseReload()
    self.wndReload:Show(false)
end

function VideoProfiles:OnReload()
    ChatSystemLib.Command("/reloadui")
end

function VideoProfiles:OnOptions()
	if self.wndMain:IsVisible() then self.TimerFPS = nil	
	    self.wndMain:Close()
	else self:OnVideoProfilesOn()
	self.TimerFPS = ApolloTimer.Create(1, true, "OnFPS", self)
	end
end

function VideoProfiles:OnFPS()
    if not self.wndMain:IsVisible() then self.TimerFPS = nil
	return end 
    local wndFPS = self.wndMain:FindChild("FPS")
	local nFPS = math.floor(GameLib.GetFrameRate() + 0.5) or 0.0
	local crColor = "green"
	if nFPS < 15 then crColor = "red" elseif nFPS < 30 then crColor = "yellow" end
	wndFPS:SetTextColor(crColor)
	wndFPS:SetText(nFPS.." fps")
end
---------------------------------- Save / Restore ---------------------------------------------------
function VideoProfiles:OnSave(eType)
    if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then return end
	local locWindowLocation = self.wndMain and self.wndMain:GetLocation() or self.locSavedWindowLoc
	local locProfilesLocation = self.wndProfiles and self.wndProfiles:GetLocation() or self.locProfilesLocation
	local tSaved = { 
	    tWindowLocation = locWindowLocation and locWindowLocation:ToTable() or nil,
		tProfilesLocation = locProfilesLocation and locProfilesLocation:ToTable() or nil, 
		bShow = self.bShow,
		bCheckRu = self.bCheckRu,
		CurrentProfile = self.nCurrentProfile,
		tProfiles = self.tProfiles
		}
	return tSaved
end

function VideoProfiles:OnRestore(eType,tSavedData)
    self.tSavedData = tSavedData
    if tSavedData.bShow ~= nil then	self.bShow = tSavedData.bShow 
	    else self.bShow = true 
	end
	if tSavedData.tProfilesLocation ~= nil then self.locProfilesLocation = WindowLocation.new(tSavedData.tProfilesLocation)
		if self.wndProfiles then self.wndProfiles:MoveToLocation(self.locProfilesLocation) end
	end
	if tSavedData.tWindowLocation ~= nil then self.locSavedWindowLoc = WindowLocation.new(tSavedData.tWindowLocation)
		if self.wndMain then self.wndMain:MoveToLocation(self.locSavedWindowLoc) end
	end
	if tonumber(tSavedData.CurrentProfile) ~= nil then self.nCurrentProfile = tSavedData.CurrentProfile
	else self.nCurrentProfile = 1
	end
	if tSavedData.bCheckRu == true or tSavedData.bCheckRu == false then self.bCheckRu = tSavedData.bCheckRu 
	else self.bCheckRu = false 
	end
	if tSavedData.tProfiles ~= nil then self.tProfiles = tSavedData.tProfiles end
end
---------------------------------- Save Profile ------------------------------------
function VideoProfiles:OnSaveProfile()
    local tCurr = {}
	for idx, name in pairs(tFrames) do
	    tCurr[name] = Apollo.GetConsoleVariable(name)
	end
	self.tProfiles[self.nCurrentProfile] = tCurr
	strMessage = "Profile '"..self.nCurrentProfile.."' saved..."
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Realm, strMessage, "")
	PL = Apollo.GetAddon("PratLite")
	if PL ~= nil then PL:OnFloatText(strMessage, 0xFFD000, -240, 7) end
end
------------------------------------- Russian Localzation --------------------------
function VideoProfiles:OnLocalizationRu()
    local tTextRussian = {}
    tTextRussian["WindowVerticalSync"]        = {text = " Верт.синхронизация/Максимальный FPS:",
    	tooltip = "Верт.синхронизация - сглаживает разницу частоты кадров в игре с частотой вертикальной развёртки монитора.\n Для правильной установки надо знать настройки Вашей видеокарты(проверить вкл/выкл). В современных играх вертикальная синхронизация рабоает хорошо. В WS - часто не справляется.\n Прим. Если не знаете что ставить - устанавливаем частоту кадров на минимальное значение FPS от значений на котором Вам комфортно играть, тогда при резких скачках FPS(увеличение и понижение в начале/конце боя) не так будет заметно дергание избражения. В этой игре это 25-40. Лучше всего сделать для профиля на производительность ручную настройку, а на другие профили установить верикальную синхронзацию."}
    tTextRussian["ViewDistanceSliderFrame"]   = {text = "Дальность видимоси:", 
	    tooltip= " Установка дальности видимости обьектов.\n- для повышения производительности: ползунок влево."}
    tTextRussian["ViewDistanceFog"]           = {text = "Горизонт:", 
	    tooltip= " Дальность прорисовки горизонта.\n- для повышения производительности: ползунок влево."}
    tTextRussian["ClutterDistance"]           = {text = "Детальность видимости", 
	    tooltip= " Устанавливает на каком максимальном расстоянии от Вас будут отображаться мелкие обьекты \n- для повышения производительности: ползунок влево."}
    tTextRussian["FieldOfVision"]             = {text = "Угол обзора:", 
	    tooltip= " Устанавливает ширину обзора камеры\n Прим. на производительность существенно не влияет."}
    tTextRussian["GammaScale"]                = {text = "Гамма:",
	    tooltip= " Осветлене/Затемнение фонового освещения."}
    tTextRussian["TextureLOD"]                = {text = "Качество тестур:", 
	    tooltip= "Уровень детализации для игровых тестур.\n Расширенная настройка(x3) от стандартной игровой. При минимальных значениях можно убрать отбражение элементов обьекта.\n- для повышения производительности: ползунок влево."}
    tTextRussian["TexFilter"]                 = {text = "Фильтрация текстур:", 
	    tooltip= "Уровень фильтрации детализации текстур.\n Для правильной установки надо знать настройки Вашей видеокарты(проверить откл/вкл Анизотропная фильрация).\n  Максимальная производительность - 'Билинейная'"}
    tTextRussian["Shadow"]                    = {text = "Динамические тени:", 
	    tooltip= " Настройка отображения детализации теней от обьектов.\n- для повышения производительности: ползунок влево."}
    tTextRussian["FXXA"]                      = {text = "Сглаживание FXAA:", 
	    tooltip= " Повышает качество изображения с наименьшим влиянием на производительность.\n Для правильной установки надо знать настройки Вашей видеокарты(проверить откл/вкл).\n- для повышения производительности: ползунок влево."}
    tTextRussian["ClutterDensity"]            = {text = "Плотность предметов:", 
	    tooltip= " Устанавливает количество мелких предметов на определенный участок экрана (количество камешков, кочек, травы и т.д.\n- для повышения производительности: ползунок влево."}
    tTextRussian["SceneDetail"]               = {text = "Размер обьекта:", 
	    tooltip= " Установка размера обьекта до полного исчезновения на расстоянии.\n- для повышения производительности: ползунок влево."}
    tTextRussian["ParticleScale"]             = {text = "Мелкие частицы:", 
	    tooltip= "Устанавливает кол-во мелких частиц на участок экрана (искры костра, блики от заклинаний и т.п.).\n- для повышения производительности: ползунок влево."}
    tTextRussian["VisualSupression"]          = {text = "Групповые эффекты от заклинаний:",
    	tooltip= "Задает количество одновременного отображения эффектов от заклинаний, когда вы находитесь в группе.\n- для повышения производительности: ползунок влево."}
    tTextRussian["LandLOD"]                   = {text = "Параллакс-эффект:",
    	tooltip= "Устанавливает уровень качества изображения недвижимых обьектов, путем более тщательной прорисовкой мелких деталей. \n- для повышения производительности: ползунок влево."}
    tTextRussian["RenderTargetScale"]         = {text = "Даунрессинг:",
    	tooltip= "Принудительное повышение производительности игры, путем снижения качества изображения. \n- для повышения производительности: ползунок влево.\n Прим. Применять в случаях, если больше никак невозможно устранить лаги, фризы, дерганья и им аналогичное."}
    tTextRussian["camera.shake"]              = {text = " Встряска камеры",
    	tooltip= "Встряска экрана при эффектах. \n - для повышения производительности: отключить"}
    tTextRussian["camera.reorient"]           = {text = "      Автовыравнивание камеры",
    	tooltip= "Возращает обзор камеры в прежнее состояние, после эффектов взрывов, вибраций и т.п. на экране"}
    tTextRussian["ShowProfiles"]              = {text = "  Панель профилей",
    	tooltip= "Отображать панель профилей"}
    tTextRussian["Ru"]                        = {text = "Ру",
    	tooltip= "Русский язык"}
	tTextRussian["SaveButton"]                = {text = "Сохранить",
    	tooltip= "Сохранить профиль"}
	for name, data in pairs(tTextRussian) do
	    if name == "Ru" or name == "ShowProfiles" or name == "camera.reorient" or name == "camera.shake" or name == "SaveButton" then self.wndMain:FindChild(name):SetTooltip(data.tooltip)
		else self.wndMain:FindChild(name):FindChild("Text"):SetText(data.text)
		    self.wndMain:FindChild(name):SetTooltipType(1)
			strToolTip = string.format("<T Font=\"CRB_Interface14_BBO\" TextColor=\"xkcdAmber\">%s</T>", data.tooltip)
		    self.wndMain:FindChild(name):SetTooltip(strToolTip)
		end
	end
end
-----------------------------------------------------------------------------------------------
-- VideoProfiles Instance
-----------------------------------------------------------------------------------------------
local VideoProfilesInst = VideoProfiles:new()
VideoProfilesInst:Init()
